//---------------------------------------------------------------------------
//
// SysUtils.h
//
// SUBSYSTEM:   Hook system
//				
// MODULE:      Hook tool
//				
// DESCRIPTION: Common utilities. 
//              Provides interface and implementation of some frequently
//              used functions
// 
//             
// AUTHOR:		Ivo Ivanov (ivopi@hotmail.com)
// DATE:		2001 December v1.00
//
//---------------------------------------------------------------------------
#ifndef _SYSUTILS_H_
#define _SYSUTILS_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Common.h"
#include "CommonStruct.h"

// OS 
#define OS_WINDOWS_95		1
#define OS_WINDOWS_98		2
#define OS_WINDOWS_ME		3
#define OS_WINDOWS_NT		4
#define OS_WINDOWS_2000		5
#define OS_WINDOWS_XP		6



//---------------------------------------------------------------------------
// IsToolHelpSupported
//
//
//---------------------------------------------------------------------------
static BOOL IsToolHelpSupported()
{

	BOOL    bResult(FALSE);
	HMODULE hModToolHelp;
	PROC    pfnCreateToolhelp32Snapshot;

	hModToolHelp = ::LoadLibraryA( "KERNEL32.DLL" );
	if (NULL != hModToolHelp)
	{
		pfnCreateToolhelp32Snapshot = ::GetProcAddress( 
			hModToolHelp,
            "CreateToolhelp32Snapshot"
			);
		bResult = (NULL != pfnCreateToolhelp32Snapshot);
		::FreeLibrary(hModToolHelp);
	} // if

	return bResult;
}


//---------------------------------------------------------------------------
// IsPsapiSupported
//
//
//---------------------------------------------------------------------------
static BOOL IsPsapiSupported()
{
	BOOL bResult = FALSE;
	HMODULE hModPSAPI = NULL;

	hModPSAPI = ::LoadLibraryA( "PSAPI.DLL" );
	bResult = (NULL != hModPSAPI);
	if (NULL != hModPSAPI)
		::FreeLibrary(hModPSAPI);

	return bResult;
}



//////////////////////////////////////////////////////////
//  Function Name  
//      GetOSVersionType
//
//  Parameters 
//
//  Return Values
//      ġ Windows  return
//		-1: Failed
//		1 : Windows 95
//		2 : Windows 98
//		3 : Windows ME
//		4 : Windows NT
//		5 : Windows 2000
//		6 : Windows XP
static int WINAPI GetOSVersionType() 
{
	UINT nOSVersion;
	OSVERSIONINFOEX osvi;
	BOOL bOsVersionInfoEx;
	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
	
	if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
	{
		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
			return -1;
	}
	
	switch (osvi.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_NT:
		// Test for the product.
		if ( osvi.dwMajorVersion <= 4 )
			nOSVersion = 4;
		else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
			nOSVersion = 5;
		else if( bOsVersionInfoEx )  
			nOSVersion = 6;
		break;
	case VER_PLATFORM_WIN32_WINDOWS:
		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
			nOSVersion = 1;
		else if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
			nOSVersion = 2;
		else if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
			nOSVersion = 3;
		break;
	}
	
	return nOSVersion; 
}
	


//---------------------------------------------------------------------------
// IsWindows9x
//
//
//---------------------------------------------------------------------------
static BOOL WINAPI IsWindows9x() 
{
	BOOL bResult = FALSE;
	OSVERSIONINFO vi = { sizeof(vi) };

	::GetVersionEx(&vi);
	if (vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) 
		bResult = TRUE;

	return bResult;
}
	

//---------------------------------------------------------------------------
// IsWindowsNT4
//
// Inspect whether the OS is Windows NT4
//---------------------------------------------------------------------------
static BOOL WINAPI IsWindowsNT4() 
{
   OSVERSIONINFO vi = { sizeof(vi) };

   ::GetVersionEx(&vi);
   
   return ( (vi.dwPlatformId == VER_PLATFORM_WIN32_NT) && 
	        (vi.dwMajorVersion == 4) );
}



//---------------------------------------------------------------------------
// ModuleFromAddress
//
// Returns the HMODULE that contains the specified memory address
//---------------------------------------------------------------------------
static HMODULE ModuleFromAddress(PVOID pv) 
{
	MEMORY_BASIC_INFORMATION mbi;

	return ((::VirtualQuery(pv, &mbi, sizeof(mbi)) != 0) 
	        ? (HMODULE) mbi.AllocationBase : NULL);
}

//---------------------------------------------------------------------------
// GetProcessHostFullName
//
// Return the path and the name of the current process
//---------------------------------------------------------------------------
static BOOL GetProcessHostFullName(char* pszFullFileName)
{
	DWORD dwResult = 0;
	::ZeroMemory((PBYTE)pszFullFileName, MAX_PATH);
	if (TRUE != ::IsBadReadPtr((PBYTE)pszFullFileName, MAX_PATH))
		dwResult = ::GetModuleFileNameA(
			NULL,                   // handle to module
			pszFullFileName,        // file name of module
			MAX_PATH                // size of buffer
			);

	return (dwResult != 0);
}


//---------------------------------------------------------------------------
// GetProcessHostName
//
// Return the name of the current process
//---------------------------------------------------------------------------
static BOOL GetProcessHostName(char* pszFullFileName)
{
	BOOL  bResult;
	char  *pdest;

	int   ch = '\\';

	bResult = GetProcessHostFullName(pszFullFileName);
	if (bResult)
	{
		// Search backward 
		pdest = strrchr(pszFullFileName, ch);
		if( pdest != NULL )
			strcpy(pszFullFileName, &pdest[1]);
	} // if

	return bResult;
}

//---------------------------------------------------------------------------
// GetProcessHostName
//
// Return the name of the current process
//---------------------------------------------------------------------------
static BOOL ExtractModuleFileName(char* pszFullFileName)
{
	BOOL  bResult = FALSE;

	if (TRUE != ::IsBadReadPtr(pszFullFileName, MAX_PATH))
	{
		char  *pdest;
		int   ch = '\\';

		// Search backward 
		pdest = strrchr(pszFullFileName, ch);
		if( pdest != NULL )
			strcpy(pszFullFileName, &pdest[1]);

		bResult = TRUE;
	} // if

	return bResult;
}

//---------------------------------------------------------------------------
// ReplaceFileName
//
//---------------------------------------------------------------------------
static BOOL ReplaceFileName(
	char* pszOldFileName,
	char* pszBaseNewFileName,
	char* pszNewFileName
	)
{
	BOOL  bResult = FALSE;
	char *pdest;
	int   ch = '\\';

	if ( 
		(TRUE != ::IsBadReadPtr(pszOldFileName, strlen(pszOldFileName) + 1))  &&
		(TRUE != ::IsBadReadPtr(pszBaseNewFileName, strlen(pszBaseNewFileName) + 1))
		)
	{
		if (TRUE != ::IsBadReadPtr(pszNewFileName, MAX_PATH))
		{
			::ZeroMemory(pszNewFileName, sizeof(MAX_PATH));
			strcpy(pszNewFileName, pszOldFileName);
			// Search backward and replaces the dll name with the hook one
			pdest = strrchr(pszNewFileName, ch);
			if( pdest != NULL )
				strcpy(&pdest[1], pszBaseNewFileName);
			else
				strcpy(pszNewFileName, pszBaseNewFileName);

			bResult = TRUE;
		} // if
	} // if

	return bResult;
}

//---------------------------------------------------------------------------
// ReplaceFileName
//
//---------------------------------------------------------------------------
static BOOL ReplaceFileName(
	HINSTANCE hOldFile,
	char*     pszBaseNewFileName,
	char*     pszNewFileName
	)
{
	BOOL  bResult = FALSE;
	char  szFileName[MAX_PATH];
	
	if (TRUE != ::IsBadReadPtr(pszBaseNewFileName, strlen(pszBaseNewFileName) + 1))
	{
		if (TRUE != ::IsBadReadPtr(pszNewFileName, MAX_PATH))
		{
			::ZeroMemory(pszNewFileName, sizeof(MAX_PATH));
			if (NULL != hOldFile)
			{
				if (0 != ::GetModuleFileNameA(hOldFile, szFileName, MAX_PATH))
					bResult = 
						ReplaceFileName(
							szFileName, 
							pszBaseNewFileName, 
							pszNewFileName);
			} // if
		} // if
	} // if

	return bResult;
}

//
// Converts a string to a boolean value
//
static BOOL StrToBool(const char* pszValue)
{
	return ( (0 == stricmp("YES", pszValue)) ||
		     (0 == stricmp("Y", pszValue)) ||
		     (0 == stricmp("TRUE", pszValue)) );
}

//
// Converts a boolean value to a string
//
static void BoolToStr(BOOL bValue, char* pszResult)
{
	bValue ? strcpy(pszResult, "Yes") : 
	         strcpy(pszResult, "No");
}


//
// Converts a Integer value to a string
//
static void IntToStr(int nValue, char* pszResult)
{
	sprintf(pszResult, "%d", nValue);
}


//
// Trims leading spaces and control characters from a string
//
static void TrimLeft(
	const char*  pszParam,
	char*        pszResult
	)
{
	char szBuffer[MAX_PATH];
	strcpy(szBuffer, "\0");
	if ( (TRUE != ::IsBadStringPtrA(pszParam, MAX_PATH)) &&
		 (strlen(pszParam) > 0) )
	{
		DWORD dwIndex = 0;
		while ( (dwIndex < strlen(pszParam)) && (pszParam[dwIndex] == ' ') )
			dwIndex++;
		if (dwIndex < strlen(pszParam))
			strcpy(szBuffer, &pszParam[dwIndex]);
	} // if
	strcpy(pszResult, szBuffer);
}
	
//
// Trims trailing spaces and control characters from a string
//
static void TrimRight(
	const char*  pszParam,
	char*        pszResult
	)
{
	char szBuffer[MAX_PATH];
	strcpy(szBuffer, "\0");
	if ( (TRUE != ::IsBadStringPtrA(pszParam, MAX_PATH)) &&
		 (strlen(pszParam) > 0) )
	{
		int nIndex = (int)strlen(pszParam) - 1;
		while ( (nIndex >= 0) && (pszParam[nIndex] == ' ') )
			nIndex--;
		if (nIndex >= 0)
		{
			memcpy(
				(PBYTE)szBuffer, 
				(PBYTE)pszParam, 
				(nIndex + 1)
				); 
			szBuffer[nIndex+1] = 0;
		} // if
	} // if
	strcpy(pszResult, szBuffer);
}

//
// Trims leading and trailing spaces and control characters from a string
//
static void Trim(
	const char*  pszParam,
	char*        pszResult
	)
{
	TrimLeft(pszParam, pszResult);
	TrimRight(pszParam, pszResult);
}



//
// Return next entry of an comma separated string
//
static BOOL GetNextCommaSeparatedString(
	const char*  pszParam,
	char*        pszResult,
	DWORD        dwLength,
	long*        pnCommaPos
	)
{
	*pnCommaPos = -1;
	BOOL   bResult = FALSE;
	char*  pdest;
	strcpy(pszResult, "\0");

	if (strlen(pszParam) > 0)
	{
		::ZeroMemory((PBYTE)pszResult, dwLength);

		pdest = strstr(pszParam, ",");
		if (pdest)
			*pnCommaPos = (char)pdest - (char)pszParam - 1;
		else
			*pnCommaPos = (long)strlen(pszParam);
		memcpy(
			(PBYTE)pszResult, 
			(PBYTE)pszParam, 
			((*pnCommaPos) + 1)
			); 
		(*pnCommaPos)++;

		Trim(pszResult, pszResult);

		bResult = TRUE;
	} // if

	return bResult;
}


//---------------------------------------------------------------------------
// UnicodeToAnsi
// 
// Tranlsates Unicode to Ansi strings
//---------------------------------------------------------------------------
static BOOL UnicodeToAnsi(
	LPWSTR pszwUniString, 
	LPSTR  pszAnsiBuff,
	DWORD  dwAnsiBuffSize
	)
{
	if(pszwUniString == NULL)
		return FALSE;

	int lendst = WideCharToMultiByte(CP_ACP, 0, pszwUniString, -1, NULL, NULL, NULL, NULL);
    if (lendst < 1)
		return FALSE;
	
	if(lendst > (int)dwAnsiBuffSize)
		lendst = dwAnsiBuffSize;

  	int nRet = WideCharToMultiByte(CP_ACP, 0, pszwUniString, -1, pszAnsiBuff, lendst, NULL, NULL);
	
	pszAnsiBuff[lendst] = '\0';
	if(nRet == 0 && dwAnsiBuffSize == -1)
		return FALSE;
	return TRUE;
}


//---------------------------------------------------------------------------
// UnicodeToAnsiEx
// 
// Tranlsates Unicode to Ansi strings
//---------------------------------------------------------------------------
static char* UnicodeToAnsiEx(LPWSTR pszwUniString, UINT cbCount)
{
	char *pszText = NULL;
	char *pszNew  = NULL;

	try
	{
		if(pszwUniString == NULL || cbCount == 0)
			return NULL;
		int lendst = WideCharToMultiByte(CP_ACP, 0, pszwUniString, -1, NULL, NULL, NULL, NULL);
		if (lendst < 1)
			return NULL;
			
		pszText = new char[lendst + 1]; 

  		int nRet = WideCharToMultiByte(CP_ACP, 0, pszwUniString, -1, pszText, lendst, NULL, NULL);
		pszText[lendst] = '\0';

		if(cbCount > 0)
		{
			int nSuccess = TRUE;
			int nCnt = 0;
			int i;

			for(i = 0; i < lendst; i++)
			{
				if( pszText[i] & 0x80)	//ѱ̸
				{
					if((i + 1) < lendst && pszText[i+1] & 0x80)
					{
						i++;
						if(cbCount == ++nCnt)
							break;
					}
					else
					{
						nSuccess = FALSE;
						break;
					}
				}
				else
				{
					if(cbCount == ++nCnt)
						break;
				}
			}

			if(nSuccess == FALSE)
			{
				delete [] pszText;
				return NULL;
			}
		
			i++;
			pszNew = new char[i + 1];
			strncpy(pszNew, pszText, i);
			pszNew[i] = '\0';

			delete [] pszText;
			return pszNew;
		}
	}
	catch(...)
	{
		if(pszNew)  delete [] pszNew;
		if(pszText) delete [] pszText;
		return NULL;
	}
	return pszText;
}


static char* GlyphToString(HDC hdc, LPWSTR lpString, int cbCount)
{	
	GCP_RESULTSW res;
	
	try
	{
		DWORD dwFontInfo = ::GetFontLanguageInfo(hdc);
		if(((dwFontInfo & GCP_GLYPHSHAPE) != GCP_GLYPHSHAPE) ||
		   ((dwFontInfo & GCP_ERROR) == GCP_ERROR))
		   return NULL;

		DWORD dwResult = ::GetCharacterPlacementW(hdc, lpString, cbCount, 0, &res, GCP_GLYPHSHAPE);
		if(dwResult == 0)
		{
			return NULL;
		}
		if(res.lpOutString == NULL)
			return NULL;
	}
	catch(...)
	{
		return NULL;
	}

	char *pText = UnicodeToAnsiEx(res.lpOutString, cbCount);
	return pText;
}


// XP϶,   ڵ ϰ ExtTextOut fuOption÷׿ ETO_IGNORELANGUAGE ԵǸ
//  ν  ʴ´()
static BOOL IsSpecialWindow(HWND hwnd)
{
	try
	{
		char ch[128];	// text buffer
		::GetClassNameA (hwnd, ch, 128);

/*		if (!stricmp(ch, __TEXT("EDIT")) || 
			!stricmp(ch, __TEXT("Static")) ||
			!stricmp(ch, __TEXT("Button")) ||
			!stricmp(ch, __TEXT("_WwG")) ||
			!stricmp(ch, __TEXT("SysTreeView32")) ||
			!stricmp(ch, __TEXT("SysListView32")) ||
			!stricmp(ch, __TEXT("DirectUIHWND")))
		{
			return TRUE;
		}
		*/
	}
	catch(...)
	{
		return FALSE;
	}
	return FALSE;
}


//   ڵ   ν(CModuleScope::ParseRecognitionText() ) ,
// 콺 y Ѵ.
//  TextOut y 0  
static BOOL IsIgnoreTopOffsetWnd(HWND hwnd)
{
	try
	{
		char ch[128];	// text buffer
		::GetClassNameA(hwnd, ch, 128);

/*		if (!stricmp(ch, __TEXT("ListBox")) ||
			!stricmp(ch, __TEXT("Button")))
		{
			return TRUE;
		}
		*/
	}
	catch(...)
	{
		return FALSE;
	}

	return FALSE;
}


static BOOL IsEqualPoint(const POINT pt1, const POINT pt2)
{
	if(pt1.x == pt2.x && pt1.y == pt2.y)
		return TRUE;
	return FALSE;
}



//---------------------------------------------------------------------------
// GetFirstLineTextEx
// 
// Tranlsates Unicode to Ansi strings
// pszText : ݵ Null Termintated  ̾!
//---------------------------------------------------------------------------
static char* GetFirstLineTextEx(char *pszText, int nSize)
{
	if(pszText == NULL || nSize == 0)
		return NULL;

	if(nSize < 0)
		nSize = (int)strlen(pszText);

	int i = 0;
	while(1)
	{
		if(i >= nSize)
			break;

		if(pszText[i] == '\r\n' || pszText[i] == '\n' || pszText[i] == '\r')
			break;
		i++;
	}

	if(i >= 1)
	{
		int nSize;
		if(i > MAX_REC_TEXT_SIZE)
			nSize = MAX_REC_TEXT_SIZE;
		else
			nSize = i;

		char *pNew = new char[nSize+1];
		strncpy(pNew, pszText, nSize);
		pNew[nSize] = '\0';
		return pNew;
	}
	return NULL;
}


//---------------------------------------------------------------------------
// IsValidCharacter
// 
// ѱ۹,  ϶ True
//---------------------------------------------------------------------------
static BOOL IsValidCharacter(char cText)
{
	if( cText & 0x80 || ::isalpha(cText))
		return TRUE;

	return FALSE;
}



#endif //_SYSUTILS_H_

//--------------------- End of the file -------------------------------------
